#include <opencv2/opencv.hpp>
#include "median_filtering.h"

using namespace cv;

char helper[] = 
"01_IPP_MedianFiltering.exe <image> [<kernel>]\n\
\t<image> - image name\n\
\t<kernel> - kernel size\n\
";

int main(int argc, char **argv)
{
    Mat srcImgOCV, srcImgIPP, dstImgOCV, dstImgIPP;
    int kDiffPoints, kSize = 5;
    if (argc < 2)
    {
        printf("%s", helper);
        return 1;
    }
    if (argc > 2)
    {
        kSize = atoi(argv[2]);
    }
    
    //  
    srcImgOCV = imread(argv[1]);
    if (srcImgOCV.data == 0)
    {
        printf("ERROR!!! imread(...)");
        return 1;
    }
    srcImgOCV.copyTo(srcImgIPP);
    
    median_opencv(srcImgOCV, dstImgOCV, kSize);
    median_ipp(srcImgIPP, dstImgIPP, kSize);
    kDiffPoints = compare(dstImgOCV, dstImgIPP);
    if (kDiffPoints == 0)
    {
        printf("OpenCV and IPP give the same result.\n");
    }
    else
    {
        printf("Number of different points %d\n", kDiffPoints);
    }

    //  ,  
	namedWindow("srcImg");
    imshow("srcImg", srcImgOCV);
    namedWindow("dstImgOCV");
    imshow("dstImgOCV", dstImgOCV);
	namedWindow("dstImgIPP");
    imshow("dstImgIPP", dstImgIPP);
    waitKey();
    
    //  
    srcImgOCV.release();
    srcImgIPP.release();
    dstImgOCV.release();
	dstImgIPP.release();
    return 0;
}