#include "median_filtering.h"

int median_opencv(const Mat &srcImg, Mat &dstImg, const int kSize)
{
    medianBlur(srcImg, dstImg, kSize);
    return 0;
}

int median_ipp(const Mat &srcImg, Mat &dstImg, const int msk)
{
    const int borderSize = msk / 2;
    IppiSize mskSize = { msk    , msk     };
    IppiPoint anchor = { msk / 2, msk / 2 };
    Mat ippSrcImg, ippDstImg;
    Size ippImgSize;
    IppiSize ippSrcSize, ippDstSize;
    Ipp8u *pSrcData, *pDstData;
    
    ippImgSize.width  = srcImg.size().width  + 2 * borderSize;
    ippImgSize.height = srcImg.size().height + 2 * borderSize;
    ippSrcImg.create(ippImgSize, srcImg.type());
    ippDstImg.create(ippImgSize, srcImg.type());
    pSrcData = (Ipp8u *)ippSrcImg.data;
    pDstData = (Ipp8u *)ippDstImg.data;

    ippSrcSize.width  = srcImg.size().width;
    ippSrcSize.height = srcImg.size().height;
    ippDstSize.width  = srcImg.size().width  + 2 * borderSize;
    ippDstSize.height = srcImg.size().height + 2 * borderSize;
    ippiCopyReplicateBorder_8u_C3R(srcImg.data, srcImg.step1(), ippSrcSize, 
        pSrcData, srcImg.step1() + 3 * 2 * borderSize, ippDstSize, borderSize, 
        borderSize);

    ippiFilterMedian_8u_C3R(
        pSrcData + ippImgSize.width * 3 * borderSize + 3 * borderSize, 
        srcImg.step1() + 3 * 2 * borderSize, 
        pDstData + ippImgSize.width * 3 * borderSize + 3 * borderSize, 
        srcImg.step1() + 3 * 2 * borderSize, ippSrcSize, mskSize, anchor);

    dstImg.create(srcImg.size(), srcImg.type());
    pSrcData = pDstData;
    pDstData = (Ipp8u *)dstImg.data;
    ippiCopy_8u_C3R(pSrcData + ippImgSize.width * 3 * borderSize + 3 * borderSize, 
        srcImg.step1() + 3 * 2 * borderSize, pDstData, dstImg.step1(), ippSrcSize);

    ippSrcImg.release();
    ippDstImg.release();

    return 0;
}

int compare(const Mat &srcImgOCV, const Mat &srcImgIPP)
{
    int kPoints = 0;
    int w = srcImgOCV.size().width, h = srcImgOCV.size().height, i, j, idx;
    for (i = 1; i < h - 1; i++)
    {
        idx = 3 * i * w;
        for (j = 1; j < w - 1; j++, idx += 3)
        {
            if (srcImgOCV.data[idx] - srcImgIPP.data[idx] != 0 ||
                srcImgOCV.data[idx + 1] - srcImgIPP.data[idx + 1] != 0 ||
                srcImgOCV.data[idx + 2] - srcImgIPP.data[idx + 2] != 0)
            {
                printf("i = %d\tj = %d\n", i, j);
                kPoints++;
            }
        }
    }
    return kPoints;
}
